#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

__all__ = ['create_order', 'initialize_payment', 'PaynovaException']

from django.conf import settings


def setting_default(name, default_value):
    value = getattr(settings, name, default_value)
    setattr(settings, name, value)

setting_default('PAYNOVA_MERCHANT_ID', None)
setting_default('PAYNOVA_PASSWORD', None)
setting_default('PAYNOVA_SECRET', None)
setting_default('PAYNOVA_LIVE', False)
setting_default('PAYNOVA_ENDPOINT', None)
setting_default('PAYNOVA_DEFAULT_LANGUAGE', 'eng')
setting_default('PAYNOVA_CALLBACK_URL', None)

from .payment import create_order, initialize_payment
from paynova_api_python_client import PaynovaException


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from preggy import expect
from tests.base import TestCase
from httmock import HTTMock
from datetime import datetime
from django.http import HttpResponse, HttpRequest, HttpResponseBadRequest, HttpResponseNotFound
from django.conf import settings
from django.dispatch import receiver
from django_paynova import create_order
from django_paynova.views import paynova_callback, paynova_cancel, paynova_pending, paynova_success
from django_paynova.models import PaynovaPayment
from django_paynova.signals import paynova_payment
from .mock import paynova_mock

import hashlib
import sys

_ver = sys.version_info


class ViewTestCase(TestCase):
    ehn_common_data_fields = None

    def _get_checksum(self, data):
        hash_string = '%s;%s;%s;%s;%s' % (
            data.get('EVENT_TYPE'),
            data.get('EVENT_TIMESTAMP'),
            data.get('DELIVERY_TIMESTAMP'),
            data.get('MERCHANT_ID'),
            settings.PAYNOVA_SECRET
        )

        if _ver >= (3, 0):
            hash_string = hash_string.encode('ascii')

        h = hashlib.sha1()
        h.update(hash_string)

        return h.hexdigest()

    def setUp(self):
        PaynovaPayment.objects.all().delete()

        self.ehn_common_data_fields = {
            'EVENT_TYPE': 'PAYMENT',
            'EVENT_TIMESTAMP': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'DELIVERY_TIMESTAMP': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'MERCHANT_ID': settings.PAYNOVA_MERCHANT_ID,
            'ORDER_ID': TestCase.ORDER_ID,
            'ORDER_NUMBER': 'order-001',
            'SESSION_ID': TestCase.ORDER_SESSION_ID,
            'PAYMENT_STATUS': 'COMPLETED',
            'PAYMENT_STATUS_REASON': 'NONE',
            'AMOUNT': '10.00',
            'CURRENCY_CODE': 'EUR',
        }

        self.ehn_common_data_fields['DIGEST'] = self._get_checksum(self.ehn_common_data_fields)

    def tearDown(self):
        pass

    def test_view(self):
        request = HttpRequest()
        expect(paynova_success(request)).to_be_instance_of(HttpResponse)
        expect(paynova_cancel(request)).to_be_instance_of(HttpResponse)
        expect(paynova_pending(request)).to_be_instance_of(HttpResponse)

    def test_callback_checksum(self):
        request = HttpRequest()
        self.ehn_common_data_fields['DIGEST'] = 'wrong-digest'
        request.POST = self.ehn_common_data_fields

        expect(paynova_callback(request)).to_be_instance_of(HttpResponseBadRequest)

    def test_callback_checksum2(self):
        request = HttpRequest()
        self.ehn_common_data_fields['DIGEST'] = self.ehn_common_data_fields['DIGEST'].upper()
        request.POST = self.ehn_common_data_fields

        expect(paynova_callback(request)).not_to_be_instance_of(HttpResponseBadRequest)

        request = HttpRequest()
        self.ehn_common_data_fields['DIGEST'] = self.ehn_common_data_fields['DIGEST'].lower()
        request.POST = self.ehn_common_data_fields

        expect(paynova_callback(request)).not_to_be_instance_of(HttpResponseBadRequest)

    def test_callback_event_type(self):
        request = HttpRequest()
        self.ehn_common_data_fields['EVENT_TYPE'] = 'UNKNOWN_EVENT_TYPE'
        self.ehn_common_data_fields['DIGEST'] = self._get_checksum(self.ehn_common_data_fields)
        request.POST = self.ehn_common_data_fields

        expect(paynova_callback(request)).to_be_instance_of(HttpResponseBadRequest)

    def test_callback_not_found(self):
        request = HttpRequest()
        request.POST = self.ehn_common_data_fields

        expect(paynova_callback(request)).to_be_instance_of(HttpResponseNotFound)

    def test_callback(self):
        with HTTMock(paynova_mock):
            create_order(self.create_order_params)

        request = HttpRequest()
        request.POST = self.ehn_common_data_fields
        expect(paynova_callback(request)).to_be_instance_of(HttpResponse)

        pp = PaynovaPayment.objects.filter().first()
        expect(pp.status).to_equal(self.ehn_common_data_fields.get('PAYMENT_STATUS'))
        expect(pp.status_reason).to_equal(self.ehn_common_data_fields.get('PAYMENT_STATUS_REASON'))
        expect(pp.params_ehn).to_equal(self.ehn_common_data_fields)

    def test_signal(self):
        with HTTMock(paynova_mock):
            create_order(self.create_order_params)

        @receiver(paynova_payment)
        def paynova_payment_signal(sender, status, params, **kwargs):
            pp = PaynovaPayment.objects.filter().first()
            expect(status).to_equal(self.ehn_common_data_fields.get('PAYMENT_STATUS'))
            expect(sender).to_equal(pp)
            expect(params).to_equal(self.ehn_common_data_fields)

        request = HttpRequest()
        request.POST = self.ehn_common_data_fields
        expect(paynova_callback(request)).to_be_instance_of(HttpResponse)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from preggy import expect
from tests.base import TestCase
from django.conf import settings
from django_paynova import settings as my_settings


class SettingsTestCase(TestCase):
    def test_settings(self):
        expect(my_settings.PAYNOVA_MERCHANT_ID).to_equal(settings.PAYNOVA_MERCHANT_ID)
        expect(my_settings.PAYNOVA_PASSWORD).to_equal(settings.PAYNOVA_PASSWORD)
        expect(my_settings.PAYNOVA_LIVE).to_equal(False)
        expect(my_settings.PAYNOVA_ENDPOINT).to_be_null()
        expect(my_settings.PAYNOVA_DEFAULT_LANGUAGE).to_length(3)
        expect(my_settings.PAYNOVA_CALLBACK_URL).not_to_be_null()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.dispatch import Signal

"""
    Signal fired when Event Hook Notification from paynova comes
"""
paynova_payment = Signal(providing_args=["status", "params", ])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.conf.urls import patterns, url

from . import views

urlpatterns = patterns('',
    url('^success/$', views.paynova_success, name='paynova_success'),
    url('^cancel/$', views.paynova_cancel, name='paynova_cancel'),
    url('^pending/$', views.paynova_pending, name='paynova_pending'),
    url('^callback/$', views.paynova_callback, name='paynova_callback'),
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from preggy import expect
from tests.base import TestCase
from django_paynova.models import PaynovaPayment


class ModelTestCase(TestCase):
    test_object = {
        'test_string': 'test',
        'test_int': 1,
        'test_array': [
            'test1', 'test2', 'test3'
        ],
        'test_object': {
            'test1': 1,
            'test2': 2
        }
    }

    def setUp(self):
        PaynovaPayment.objects.all().delete()

    def tearDown(self):
        pass

    def test_save_empty_model(self):
        pp = PaynovaPayment()
        pp.save()
        expect(pp).to_be_instance_of(PaynovaPayment)

    def test_params_create_order(self):
        pp = PaynovaPayment()
        pp.params_create_order = self.test_object
        pp.save()

        pp2 = PaynovaPayment.objects.filter().first()

        expect(pp2.params_create_order).to_equal(self.test_object)

    def test_params_init_payment(self):
        pp = PaynovaPayment()
        pp.params_init_payment = self.test_object
        pp.save()

        pp2 = PaynovaPayment.objects.filter().first()

        expect(pp2.params_init_payment).to_equal(self.test_object)

    def test_params_ehn(self):
        pp = PaynovaPayment()
        pp.params_ehn = self.test_object
        pp.save()

        pp2 = PaynovaPayment.objects.filter().first()

        expect(pp2.params_ehn).to_equal(self.test_object)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.conf import settings
from .models import PaynovaPayment
from paynova_api_python_client import Paynova, PaynovaException
from django.core.urlresolvers import reverse


def create_order(params, init_payment=True):
    """
        Create order
        :param params: See docs for params
        :param init_payment: If True - auto init payment.
        :return: PaynovaPayment model

        Docs: http://docs.paynova.com/display/API/Create+Order
    """

    # create a model

    model = PaynovaPayment()
    model.params_create_order = params
    model.save()

    # make request

    client = Paynova(settings.PAYNOVA_MERCHANT_ID, settings.PAYNOVA_PASSWORD,
                     live=settings.PAYNOVA_LIVE, endpoint=settings.PAYNOVA_ENDPOINT)

    try:
        response = client.create_order(params)
    except PaynovaException as e:
        model.status = PaynovaPayment.STATUS_ERROR
        model.status_reason = '%s' % e
        model.save()
        raise e

    model.order_id = response.get('orderId')
    model.status = PaynovaPayment.STATUS_ORDER_CREATED
    model.save()

    # make init with defaults

    if init_payment:
        return initialize_payment({'orderId': model.order_id})

    return model


def _get_url_params():
    """
        Build urls for callbacks
        Docs: http://docs.paynova.com/display/API/Initialize+Payment
    """
    if not settings.PAYNOVA_CALLBACK_URL:
        raise PaynovaException({'statusMessage': 'PAYNOVA_CALLBACK_URL should be defined in settings'})

    url = settings.PAYNOVA_CALLBACK_URL.strip('/')

    return {
        'urlRedirectSuccess': '%s%s' % (url, reverse('paynova_success')),
        'urlRedirectCancel': '%s%s' % (url, reverse('paynova_cancel')),
        'urlRedirectPending': '%s%s' % (url, reverse('paynova_pending')),
        'urlCallback': '%s%s' % (url, reverse('paynova_callback')),
    }


def _get_params_for_initialize_payment(params):
    """
        Get params and model
    """
    order_id = params.get('orderId')
    if not order_id:
        raise PaynovaException({'statusMessage': 'Parameter "orderId" must be defined'})
    try:
        model = PaynovaPayment.objects.get(order_id=order_id)
    except PaynovaPayment.DoesNotExist:
        raise PaynovaException({'statusMessage': 'Order with "orderId" = %s was not found in model' % order_id})

    # set default params

    return_params = {
        'orderId': order_id,
        'totalAmount': model.params_create_order.get('totalAmount'),
        'paymentChannelId': 1,
        'interfaceOptions': {
            'interfaceId': 5,
            'customerLanguageCode': settings.PAYNOVA_DEFAULT_LANGUAGE,
        }
    }

    return_params.update(params)
    return_params['interfaceOptions'].update(_get_url_params())

    return return_params, model


def initialize_payment(params):
    """
        Initializa payment
        :param params: See docs
        :return: PaynovaPayment model

        Docs: http://docs.paynova.com/display/API/Initialize+Payment
    """

    # get model and params based on passed data

    params, model = _get_params_for_initialize_payment(params)

    model.params_init_payment = params
    model.save()

    # make request

    client = Paynova(settings.PAYNOVA_MERCHANT_ID, settings.PAYNOVA_PASSWORD,
                     live=settings.PAYNOVA_LIVE, endpoint=settings.PAYNOVA_ENDPOINT)

    try:
        response = client.initialize_payment(params)
    except PaynovaException as e:
        model.status = PaynovaPayment.STATUS_ERROR
        model.status_reason = '%s' % e
        model.save()
        raise e

    model.session_id = response.get('sessionId')
    model.url = response.get('url')
    model.status = PaynovaPayment.STATUS_PAYMENT_INITED
    model.save()

    return model


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from setuptools import setup

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
    'django_nose',
    'httmock',
    'pypandoc',
]

setup(
    name='django-paynova',
    version='0.1.4',
    description='Python client for Paynova\'s API. For Django framework.',
    long_description=read_md('README.md'),
    keywords='Paynova API client python django',
    author='Andrey Kolpakov',
    author_email='aakolpakov@gmail.com',
    url='https://github.com/akolpakov/django-paynova',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
    ],
    packages=['django_paynova'],
    include_package_data=True,
    install_requires=[
        # add your dependencies here
        # remember to use 'package-name>=x.y.z,<x.y+1.0' notation (this way you get bugfixes)
        'django>=1.6,<1.8',
        'paynova-api-python-client',
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'django-paynova=django_paynova.cli:main',
        ],
    },
)
